package net.mosgrom.modvid.gamelib;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;

/**
 * A Ship that can move around in Space.
 * 
 * @author jasper
 *
 */
public class Ship {

	private Point location;
	private Engine engine;
	
	/**
	 * Create a new Ship at coordinates (0, 0).
	 */
	public Ship() {
		location = new Point(0, 0);
		engine = new Engine(this);
	}
	
	/**
	 * Called when this Ship is added to a Space.
	 * 
	 * @param s The Space this Ship was added to.
	 */
	public void shipAdded(Space s) {
	}

	/**
	 * Update the position of this Ship and draw it.
	 * 
	 * @param g The Graphics to use for drawing.
	 */
	public void update(Graphics g) {
		updateLocation();
		g.setColor(Color.white);
		g.fillRect((int)location.getX(), (int)location.getY(), 32, 32);
	}

	/**
	 * @return The coordinates of where this Ship was the last time its position was updated.
	 */	
	public Point location() {
		return location;
	}

	/**
	 * Set course to <code>destination</code> at <code>speed</code> pixels per millisecond.
	 * @param destination Coordinates of the destination.
	 * @param speed The traveling speed in pixels per millisecond.
	 */	
	public void setCourse(Point destination, double speed) {
		engine.setCourse(destination, speed);
	}
	
	/** 
	 * @return <b>true</b> if the destination of the last set course has been reached (false otherwise).
	 */	
	public boolean destinationReached() {
		return engine.destinationReached();
	}
	
	/**
	 * Updates the coordinates returned by location() to the latest position of this Ship.
	 * This is meant to be called from update(Graphics g) before this Ship draws itself.
	 */
	protected void updateLocation() {
		location = engine.currentLocation();
	}
	
}
