package net.mosgrom.modvid.gamelib;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferStrategy;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;


/**
 * The main game world: a Space in which Ships can move around.
 * You can add Ships to it by calling addShip(Ship).
 * 
 * @author mosgrom
 *
 */
@SuppressWarnings("serial")
public class Space extends Canvas {

	/**
	 * true if update-loop is running.
	 */
	private boolean run;
	
	/**
	 * List of Ships in the this Space.
	 */
	private List<Ship> ships;
	
	/**
	 * The BufferStrategy used in the update-loop.
	 */
	private BufferStrategy buf;
	
	/**
	 * Creates a Space and a Window to show it in.
	 */
	public Space() {
		ships = new ArrayList<>();
		setBackground(Color.BLACK);
		Frame f = new Frame();
		f.setSize(1000, 600);
		f.add(this);
		f.addWindowListener(new WindowAdapter() {

			@Override
			public void windowClosing(WindowEvent e) {
				run = false;
				f.dispose();
			}
			
		});
		f.setVisible(true);
		createBufferStrategy(2);
		buf = getBufferStrategy();
		run = true;
		new Thread(this::run).start();		
	}

	/**
	 * Add a Ship to this Space.
	 * 
	 * @param s The Ship to add.
	 */
	public void addShip(Ship s) {
		ships.add(s);
		s.shipAdded(this);
	}

	/**
	 * @return Random coordinates in this Space.
	 */
	public Point randomLocation() {
		Random r = new Random();
		return new Point(r.nextInt(getWidth()), r.nextInt(getHeight()));
	}
		
	private void updateAll(Graphics g) {
		g.setColor(getBackground());
		g.fillRect(0, 0, getWidth(), getHeight());
		for (Ship s : ships) {
			s.update(g);
		}
	}
	
	private void run() {
		while (run) {
			do {
				do {
					Graphics graphics = buf.getDrawGraphics();
					updateAll(graphics);
					graphics.dispose();
		         } while (buf.contentsRestored());
		         buf.show();
		     } while (buf.contentsLost());			
		}
	}
	
}
